import BaseElement from '@emanon/shared/components/base';
import { customElement, html, property, PropertyValues } from 'lit-element';
import { style } from './index.gen.css';
import { style as darkStyle } from './dark.gen.css';
import { style as lightStyle } from './light.gen.css';
import prismjs from 'prismjs';

// 追加する言語
import 'prismjs/components/prism-jsx.min.js';
import 'prismjs/components/prism-tsx.min.js';
import 'prismjs/components/prism-typescript.min.js';

// @see https://prismjs.com/#supported-languages
const LANGUAGES = {
  '': 'Default',
  // prismjsのデフォルト言語
  markup: 'Markup(HTML, XML, SVG)',
  css: 'CSS',
  clike: 'C-like',
  javascript: 'JavaScript',
  // 追加する言語(AtoZ)
  jsx: 'React JSX',
  tsx: 'React TSX',
  typescript: 'TypeScript'
} as const;

/**
 * ## Overall
 * Highlighting <pre> & <code> block
 */
@customElement('ema-code-beautifier')
export class CodeBeautifier extends BaseElement {
  static get is() {
    return 'ema-code-beautifier';
  }

  @property({ type: String })
  value = '';

  @property({ type: String, reflect: true })
  language: keyof typeof LANGUAGES = '';

  @property({ type: String, reflect: true })
  color: 'light' | 'dark' = 'dark';

  static get styles() {
    return [super.styles, style];
  }

  static get languages() {
    return LANGUAGES;
  }

  constructor() {
    super();
  }

  updated(changedProperties: PropertyValues) {
    super.updated(changedProperties);
    if (!this.shadowRoot) {
      return;
    }
    if (!changedProperties.has('language')) {
      return;
    }
    const elm = this.shadowRoot.querySelector('pre code');
    if (elm) {
      prismjs.highlightElement(elm);
    }
  }

  private get highlightTheme(): string {
    if (this.color === 'dark') {
      return darkStyle.cssText;
    }
    if (this.color === 'light') {
      return lightStyle.cssText;
    }
    return '';
  }

  render() {
    const {
      value,
      language,
      highlightTheme
    } = this;

    return html`
      <style>${highlightTheme}</style>
      <pre><code class="language-${language}">${value}</code></pre>
    `;
  }
};
